<?php
/**
 * @file
 * Forms for the PGP Contact Forms module
 */

/**
 * Upload form for the public key
 */
function pgp_contact_pubkey(&$form_state, $type, $arg2 = NULL) {

  $mpath = drupal_get_path('module', 'pgp_contact');

  drupal_add_js(variable_get('pgp_contact_library_path', 'sites/all/libraries') . '/sha1.js');
  drupal_add_js(variable_get('pgp_contact_library_path', 'sites/all/libraries') . '/base64.js');
  drupal_add_js(variable_get('pgp_contact_library_path', 'sites/all/libraries') . '/PGpubkey.js');
  drupal_add_js($mpath . '/js/pgp_contact.key_info.js');
  drupal_add_css($mpath . '/css/pgp_contact.key_info.css');

  $form = array(
    '#tree' => TRUE,
  );

  switch ($type) {
    case 'user':
      $id = $arg2->uid;
      $form['uid'] = array(
        '#type' => 'hidden',
        '#value' => $id,
      );
      break;
    case 'contact':
      $id = NULL;
      break;
  }

  $form['type'] = array(
    '#type' => 'hidden',
    '#value' => $type,
  ); 

  // Fetch info about the key
  $key_info = _pgp_contact_user_key($id, $type);

  $form['public_key'] = array(
    '#type' => 'textarea',
    '#title' => t('Paste your public key here'),
    '#required' => TRUE,
    '#description' => t('This will not work without javascript enabled.'),
    '#default_value' => ($key_info) ? $key_info->public_key : t('Paste your public key here.'),
    '#rows' => 20,
  );

  if (variable_get('pgp_contact_sitewide_policy', 'user') == 'user') {
    $form['key_policy'] = _pgp_contact_policy_field();
    if ($key_info) {
      $form['key_policy']['pgp_contact_policy']['#default_value'] = $key_info->policy;
    }
  }

  $form['verify'] = array(
    '#type' => 'button',
    '#value' => t('Verify key'),
  );

  $form['key_info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Key Information'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#prefix' => '<div id="pgp-contact-dynamic-info">',
    '#suffix' => '</div>',
  );

  $form['key_info']['user'] = array(
    '#type' => 'textfield',
    '#title' => t('Key account'),
    '#size' => 50,
    '#maxlength' => 128,
  );

  $form['key_info']['key_type'] = array(
    '#type' => 'textfield',
    '#title' => t('Key type'),
    '#size' => 20,
    '#maxlength' => 20,
    '#required' => TRUE
  );

  $form['key_info']['hash'] = array(
    '#type' => 'textfield',
    '#title' => t('Key hash'),
    '#description' => t('Not your usual fingerprint. Check !link for a corroborating source.', array('!link' => l('http://www.hanewin.net/encrypt/PGpubkey.htm', 'http://www.hanewin.net/encrypt/PGpubkey.htm'))),
    '#size' => 50,
    '#maxlength' => 50,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit key'),
  );

  return $form;
}

/** 
 * Validate function for public key submission. 
 */ 
function pgp_contact_pubkey_validate($form, &$form_state) {

  $account = new stdClass();
  $account->uid = check_plain($form_state['values']['uid']);

  // Make sure the user has the right to change this key
  switch ($form_state['values']['type']) {
    case 'user':
      if (user_edit_access($account)) {
        break;
      }
    case 'contact':
      if (user_access('administer site-wide contact form')) {
        break;
      }
    default:
      form_set_error('type', 'Access error.');
      break;
  }

  // Make sure the key is of a supported type
  if (check_plain($form_state['values']['key_info']['type']) != 'RSA' && check_plain($form_state['values']['key_info']['key_type']) != 'ELGAMAL') { 
    form_set_error('key_type', 'Invalid key type.'); 
  } 
} 
 
/** 
 * Submit function for public key submission. 
 */ 
function pgp_contact_pubkey_submit($form, &$form_state) { 
 
  // Prepare the form values for submission 
  $record = new stdClass(); 
  $record->uid = check_plain($form_state['values']['uid']);
  $record->type = check_plain($form_state['values']['type']); 
  $record->key_type = check_plain($form_state['values']['key_info']['key_type']); 
  $record->public_key = check_plain($form_state['values']['public_key']); 
  if (variable_get('pgp_contact_sitewide_policy', 'user') == 'user') { 
    $record->policy = check_plain($form_state['values']['pgp_contact_policy']); 
  } 
  else { 
    $record->policy = 'always'; 
  } 
 
  // Check if this is an update or an insert 
  $update = array(); 
  switch ($record->type) {
    case 'user':
    if (_pgp_contact_user_key($record->uid)) { 
      $update[] = 'uid'; 
    }
    break;
    case 'contact':
    if (_pgp_contact_user_key(NULL, 'contact')) {
      $update[] = 'type';
    }
  }
  drupal_write_record('pgp_contact_pub_keys', $record, $update); 
  drupal_set_message('Public key successfully uploaded.'); 
}
